@@ -14,12 +14,7 @@
 #define MAINWINDOW_H_CQ
 
 #include <QtGlobal>
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QtWidgets>
-#else
-#include <QtGui>
-#endif
-
 
  namespace Ui {
      class MainWindow;
@@ -69,7 +64,8 @@ void printHelpAndExit(QString str);
 
 protected:
 virtual void resizeEvent(QResizeEvent* event);
-virtual void paintEvent(QPaintEvent *event);
+virtual void paintEvent(QPaintEvent* event);
+virtual void closeEvent(QCloseEvent* event);
 
 };
 
