@@ -6,7 +6,7 @@
 // Author: Jason Haslam
 //
 // Additions Copyright (c) 2011 Archaeopteryx Software, Inc. d/b/a Wingware
-// ScintillaEditBase.cpp - Qt widget that wraps ScintillaQt and provides events and scrolling
+// @file ScintillaEditBase.cpp - Qt widget that wraps ScintillaQt and provides events and scrolling
 
 #include "ScintillaEditBase.h"
 #include "ScintillaQt.h"
@@ -21,13 +21,10 @@
 #include <QScrollBar>
 #include <QTextFormat>
 
-#define INDIC_INPUTMETHOD 24
-
-#define MAXLENINPUTIME 200
-#define SC_INDICATOR_INPUT INDICATOR_IME
-#define SC_INDICATOR_TARGET INDICATOR_IME+1
-#define SC_INDICATOR_CONVERTED INDICATOR_IME+2
-#define SC_INDICATOR_UNKNOWN INDICATOR_IME_MAX
+constexpr int IndicatorInput = static_cast<int>(Scintilla::IndicatorNumbers::Ime);
+constexpr int IndicatorTarget = IndicatorInput + 1;
+constexpr int IndicatorConverted = IndicatorInput + 2;
+constexpr int IndicatorUnknown = IndicatorInput + 3;
 
 // Q_WS_MAC and Q_WS_X11 aren't defined in Qt5
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
@@ -40,21 +37,12 @@
 #endif
 #endif // QT_VERSION >= 5.0.0
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
-#define ISWHEELEVENTHORIZONTAL(x)     (x->angleDelta().y() == 0)
-#define WHEELEVENTYDELTA(x)           (x->angleDelta().y())
-#else
-#define ISWHEELEVENTHORIZONTAL(x)     (x->orientation() == Qt::Horizontal)
-#define WHEELEVENTYDELTA(x)           (x->delta())
-#endif
-
 using namespace Scintilla;
+using namespace Scintilla::Internal;
 
 ScintillaEditBase::ScintillaEditBase(QWidget *parent)
-: QAbstractScrollArea(parent), sqt(nullptr), preeditPos(-1), wheelDelta(0)
+: QAbstractScrollArea(parent), sqt(new ScintillaQt(this)), preeditPos(-1), wheelDelta(0)
 {
-	sqt = new ScintillaQt(this);
-
 	time.start();
 
 	// Set Qt defaults.
@@ -68,13 +56,13 @@ ScintillaEditBase::ScintillaEditBase(QWidget *parent)
 	setAttribute(Qt::WA_KeyCompression);
 	setAttribute(Qt::WA_InputMethodEnabled);
 
-	sqt->vs.indicators[SC_INDICATOR_UNKNOWN] = Indicator(INDIC_HIDDEN, ColourDesired(0, 0, 0xff));
-	sqt->vs.indicators[SC_INDICATOR_INPUT] = Indicator(INDIC_DOTS, ColourDesired(0, 0, 0xff));
-	sqt->vs.indicators[SC_INDICATOR_CONVERTED] = Indicator(INDIC_COMPOSITIONTHICK, ColourDesired(0, 0, 0xff));
-	sqt->vs.indicators[SC_INDICATOR_TARGET] = Indicator(INDIC_STRAIGHTBOX, ColourDesired(0, 0, 0xff));
+	sqt->vs.indicators[IndicatorUnknown] = Indicator(IndicatorStyle::Hidden, colourIME);
+	sqt->vs.indicators[IndicatorInput] = Indicator(IndicatorStyle::Dots, colourIME);
+	sqt->vs.indicators[IndicatorConverted] = Indicator(IndicatorStyle::CompositionThick, colourIME);
+	sqt->vs.indicators[IndicatorTarget] = Indicator(IndicatorStyle::StraightBox, colourIME);
 
-	connect(sqt, SIGNAL(notifyParent(SCNotification)),
-	        this, SLOT(notifyParent(SCNotification)));
+	connect(sqt, SIGNAL(notifyParent(Scintilla::NotificationData)),
+		this, SLOT(notifyParent(Scintilla::NotificationData)));
 
 	// Connect scroll bars.
 	connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
@@ -95,21 +83,21 @@ ScintillaEditBase::ScintillaEditBase(QWidget *parent)
 	connect(sqt, SIGNAL(notifyChange()),
 	        this, SIGNAL(notifyChange()));
 
-	connect(sqt, SIGNAL(command(uptr_t, sptr_t)),
-	        this, SLOT(event_command(uptr_t, sptr_t)));
+	connect(sqt, SIGNAL(command(Scintilla::uptr_t,Scintilla::sptr_t)),
+		this, SLOT(event_command(Scintilla::uptr_t,Scintilla::sptr_t)));
 
-	connect(sqt, SIGNAL(aboutToCopy(QMimeData *)),
-	        this, SIGNAL(aboutToCopy(QMimeData *)));
+	connect(sqt, SIGNAL(aboutToCopy(QMimeData*)),
+		this, SIGNAL(aboutToCopy(QMimeData*)));
 }
 
-ScintillaEditBase::~ScintillaEditBase() {}
+ScintillaEditBase::~ScintillaEditBase() = default;
 
 sptr_t ScintillaEditBase::send(
 	unsigned int iMessage,
 	uptr_t wParam,
 	sptr_t lParam) const
 {
-	return sqt->WndProc(iMessage, wParam, lParam);
+	return sqt->WndProc(static_cast<Message>(iMessage), wParam, lParam);
 }
 
 sptr_t ScintillaEditBase::sends(
@@ -117,7 +105,7 @@ sptr_t ScintillaEditBase::sends(
     uptr_t wParam,
     const char *s) const
 {
-	return sqt->WndProc(iMessage, wParam, (sptr_t)s);
+	return sqt->WndProc(static_cast<Message>(iMessage), wParam, reinterpret_cast<sptr_t>(s));
 }
 
 void ScintillaEditBase::scrollHorizontal(int value)
@@ -156,30 +144,42 @@ void ScintillaEditBase::paintEvent(QPaintEvent *event)
 	sqt->PartialPaint(PRectFromQRect(event->rect()));
 }
 
+namespace {
+
+bool isWheelEventHorizontal(QWheelEvent *event) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+	return event->angleDelta().y() == 0;
+#else
+	return event->orientation() == Qt::Horizontal;
+#endif
+}
+
+int wheelEventYDelta(QWheelEvent *event) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+	return event->angleDelta().y();
+#else
+	return event->delta();
+#endif
+}
+
+}
+
 void ScintillaEditBase::wheelEvent(QWheelEvent *event)
 {
-	if (ISWHEELEVENTHORIZONTAL(event)) {
-		if (horizontalScrollBarPolicy() == Qt::ScrollBarAlwaysOff)
-			event->ignore();
-		else
-			QAbstractScrollArea::wheelEvent(event);
+	if (isWheelEventHorizontal(event)) {
+		QAbstractScrollArea::wheelEvent(event);
 	} else {
 		if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
 			// Zoom! We play with the font sizes in the styles.
 			// Number of steps/line is ignored, we just care if sizing up or down
-			if (WHEELEVENTYDELTA(event) > 0) {
-				sqt->KeyCommand(SCI_ZOOMIN);
+			if (wheelEventYDelta(event) > 0) {
+				sqt->KeyCommand(Message::ZoomIn);
 			} else {
-				sqt->KeyCommand(SCI_ZOOMOUT);
+				sqt->KeyCommand(Message::ZoomOut);
 			}
 		} else {
-			// Ignore wheel events when the scroll bars are disabled.
-			if (verticalScrollBarPolicy() == Qt::ScrollBarAlwaysOff) {
-				event->ignore();
-			} else {
-				// Scroll
-				QAbstractScrollArea::wheelEvent(event);
-			}
+			// Scroll
+			QAbstractScrollArea::wheelEvent(event);
 		}
 	}
 }
@@ -246,8 +246,8 @@ void ScintillaEditBase::keyPressEvent(QKeyEvent *event)
 	bool alt   = QApplication::keyboardModifiers() & Qt::AltModifier;
 
 	bool consumed = false;
-	bool added = sqt->KeyDownWithModifiers(key,
-					       ScintillaQt::ModifierFlags(shift, ctrl, alt),
+	bool added = sqt->KeyDownWithModifiers(static_cast<Keys>(key),
+					       ModifierFlags(shift, ctrl, alt),
 					       &consumed) != 0;
 	if (!consumed)
 		consumed = added;
@@ -266,8 +266,15 @@ void ScintillaEditBase::keyPressEvent(QKeyEvent *event)
 
 		QString text = event->text();
 		if (input && !text.isEmpty() && text[0].isPrint()) {
-			QByteArray utext = sqt->BytesForDocument(text);
-			sqt->InsertCharacter(utext.data(), utext.size(), EditModel::CharacterSource::directInput);
+			const int strLen = text.length();
+			for (int i = 0; i < strLen;) {
+				const int ucWidth = text.at(i).isHighSurrogate() ? 2 : 1;
+				const QString oneCharUTF16 = text.mid(i, ucWidth);
+				const QByteArray oneChar = sqt->BytesForDocument(oneCharUTF16);
+
+				sqt->InsertCharacter(std::string_view(oneChar.data(), oneChar.length()), CharacterSource::DirectInput);
+				i += ucWidth;
+			}
 		} else {
 			event->ignore();
 		}
@@ -276,7 +283,6 @@ void ScintillaEditBase::keyPressEvent(QKeyEvent *event)
 	emit keyPressed(event);
 }
 
-#ifdef Q_WS_X11
 static int modifierTranslated(int sciModifier)
 {
 	switch (sciModifier) {
@@ -292,7 +298,6 @@ static int modifierTranslated(int sciModifier)
 			return 0;
 	}
 }
-#endif
 
 void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 {
@@ -300,7 +305,7 @@ void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 
 	emit buttonPressed(event);
 
-	if (event->button() == Qt::MidButton &&
+	if (event->button() == Qt::MiddleButton &&
 	    QApplication::clipboard()->supportsSelection()) {
 		SelectionPosition selPos = sqt->SPositionFromLocation(
 					pos, false, false, sqt->UserVirtualSpace());
@@ -313,15 +318,9 @@ void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 	if (event->button() == Qt::LeftButton) {
 		bool shift = QApplication::keyboardModifiers() & Qt::ShiftModifier;
 		bool ctrl  = QApplication::keyboardModifiers() & Qt::ControlModifier;
-#ifdef Q_WS_X11
-		// On X allow choice of rectangular modifier since most window
-		// managers grab alt + click for moving windows.
 		bool alt   = QApplication::keyboardModifiers() & modifierTranslated(sqt->rectangularSelectionModifier);
-#else
-		bool alt   = QApplication::keyboardModifiers() & Qt::AltModifier;
-#endif
 
-		sqt->ButtonDownWithModifiers(pos, time.elapsed(), ScintillaQt::ModifierFlags(shift, ctrl, alt));
+		sqt->ButtonDownWithModifiers(pos, time.elapsed(), ModifierFlags(shift, ctrl, alt));
 	}
 
 	if (event->button() == Qt::RightButton) {
@@ -331,12 +330,12 @@ void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 
 void ScintillaEditBase::mouseReleaseEvent(QMouseEvent *event)
 {
-	Point point = PointFromQPoint(event->pos());
+	const QPoint point = event->pos();
 	if (event->button() == Qt::LeftButton)
-		sqt->ButtonUpWithModifiers(point, time.elapsed(), ModifiersOfKeyboard());
+		sqt->ButtonUpWithModifiers(PointFromQPoint(point), time.elapsed(), ModifiersOfKeyboard());
 
-	int pos = send(SCI_POSITIONFROMPOINT, point.x, point.y);
-	int line = send(SCI_LINEFROMPOSITION, pos);
+	const sptr_t pos = send(SCI_POSITIONFROMPOINT, point.x(), point.y());
+	const sptr_t line = send(SCI_LINEFROMPOSITION, pos);
 	int modifiers = QApplication::keyboardModifiers();
 
 	emit textAreaClicked(line, modifiers);
@@ -355,15 +354,9 @@ void ScintillaEditBase::mouseMoveEvent(QMouseEvent *event)
 
 	bool shift = QApplication::keyboardModifiers() & Qt::ShiftModifier;
 	bool ctrl  = QApplication::keyboardModifiers() & Qt::ControlModifier;
-#ifdef Q_WS_X11
-	// On X allow choice of rectangular modifier since most window
-	// managers grab alt + click for moving windows.
 	bool alt   = QApplication::keyboardModifiers() & modifierTranslated(sqt->rectangularSelectionModifier);
-#else
-	bool alt   = QApplication::keyboardModifiers() & Qt::AltModifier;
-#endif
 
-	const int modifiers = ScintillaQt::ModifierFlags(shift, ctrl, alt);
+	const KeyMod modifiers = ModifierFlags(shift, ctrl, alt);
 
 	sqt->ButtonMoveWithModifiers(pos, time.elapsed(), modifiers);
 }
@@ -377,6 +370,9 @@ void ScintillaEditBase::contextMenuEvent(QContextMenuEvent *event)
 	}
 	if (sqt->ShouldDisplayPopup(pt)) {
 		sqt->ContextMenu(pos);
+		event->accept();
+	} else {
+		event->ignore();
 	}
 }
 
@@ -432,7 +428,7 @@ void ScintillaEditBase::dropEvent(QDropEvent *event)
 
 bool ScintillaEditBase::IsHangul(const QChar qchar)
 {
-	int unicode = (int)qchar.unicode();
+	unsigned int unicode = qchar.unicode();
 	// Korean character ranges used for preedit chars.
 	// http://www.programminginkorean.com/programming/hangul-in-unicode/
 	const bool HangulJamo = (0x1100 <= unicode && unicode <= 0x11FF);
@@ -444,11 +440,11 @@ bool ScintillaEditBase::IsHangul(const QChar qchar)
 				HangulJamoExtendedA || HangulJamoExtendedB;
 }
 
-void ScintillaEditBase::MoveImeCarets(int offset)
+void ScintillaEditBase::MoveImeCarets(Scintilla::Position offset)
 {
 	// Move carets relatively by bytes
 	for (size_t r=0; r < sqt->sel.Count(); r++) {
-		int positionInsert = sqt->sel.Range(r).Start().Position();
+		const Sci::Position positionInsert = sqt->sel.Range(r).Start().Position();
 		sqt->sel.Range(r).caret.SetPosition(positionInsert + offset);
 		sqt->sel.Range(r).anchor.SetPosition(positionInsert + offset);
  	}
@@ -460,12 +456,12 @@ void ScintillaEditBase::DrawImeIndicator(int indicator, int len)
 	// Draw an indicator on the character before caret by the character bytes of len
 	// so it should be called after InsertCharacter().
 	// It does not affect caret positions.
-	if (indicator < 8 || indicator > INDICATOR_MAX) {
+	if (indicator < INDICATOR_CONTAINER || indicator > INDICATOR_MAX) {
 		return;
 	}
 	sqt->pdoc->DecorationSetCurrentIndicator(indicator);
 	for (size_t r=0; r< sqt-> sel.Count(); r++) {
-		int positionInsert = sqt->sel.Range(r).Start().Position();
+		const Sci::Position positionInsert = sqt->sel.Range(r).Start().Position();
 		sqt->pdoc->DecorationFillRange(positionInsert - len, 1, len);
 	}
 }
@@ -481,40 +477,38 @@ static int GetImeCaretPos(QInputMethodEvent *event)
 
 static std::vector<int> MapImeIndicators(QInputMethodEvent *event)
 {
-	std::vector<int> imeIndicator(event->preeditString().size(), SC_INDICATOR_UNKNOWN);
+	std::vector<int> imeIndicator(event->preeditString().size(), IndicatorUnknown);
 	foreach (QInputMethodEvent::Attribute attr, event->attributes()) {
 		if (attr.type == QInputMethodEvent::TextFormat) {
 			QTextFormat format = attr.value.value<QTextFormat>();
 			QTextCharFormat charFormat = format.toCharFormat();
 
-			int indicator = SC_INDICATOR_UNKNOWN;
+			int indicator = IndicatorUnknown;
 			switch (charFormat.underlineStyle()) {
 				case QTextCharFormat::NoUnderline: // win32, linux
-					indicator = SC_INDICATOR_TARGET;
-					break;
 				case QTextCharFormat::SingleUnderline: // osx
 				case QTextCharFormat::DashUnderline: // win32, linux
-					indicator = SC_INDICATOR_INPUT;
+					indicator = IndicatorInput;
 					break;
 				case QTextCharFormat::DotLine:
 				case QTextCharFormat::DashDotLine:
 				case QTextCharFormat::WaveUnderline:
 				case QTextCharFormat::SpellCheckUnderline:
-					indicator = SC_INDICATOR_CONVERTED;
+					indicator = IndicatorConverted;
 					break;
 
 				default:
-					indicator = SC_INDICATOR_UNKNOWN;
+					indicator = IndicatorUnknown;
 			}
 
 			if (format.hasProperty(QTextFormat::BackgroundBrush)) // win32, linux
-				indicator = SC_INDICATOR_TARGET;
+				indicator = IndicatorTarget;
 
 #ifdef Q_OS_OSX
 			if (charFormat.underlineStyle() == QTextCharFormat::SingleUnderline) {
 				QColor uc = charFormat.underlineColor();
 				if (uc.lightness() < 2) { // osx
-					indicator = SC_INDICATOR_TARGET;
+					indicator = IndicatorTarget;
 				}
 			}
 #endif
@@ -547,24 +541,36 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 	}
 
 	sqt->view.imeCaretBlockOverride = false;
+	preeditPos = -1; // reset not to interrupt Qt::ImCursorRectangle.
+
+	const int rpLength = event->replacementLength();
+	if (rpLength != 0) {
+		// Qt has called setCommitString().
+		// Make room for the string to sit in.
+		const int rpStart = event->replacementStart();
+		const Scintilla::Position rpBase = sqt->CurrentPosition();
+		const Scintilla::Position start = sqt->pdoc->GetRelativePositionUTF16(rpBase, rpStart);
+		const Scintilla::Position end = sqt->pdoc->GetRelativePositionUTF16(start, rpLength);
+		sqt->pdoc->DeleteChars(start, end - start);
+	}
 
 	if (!event->commitString().isEmpty()) {
-		const QString commitStr = event->commitString();
-		const unsigned int commitStrLen = commitStr.length();
+		const QString &commitStr = event->commitString();
+		const int commitStrLen = commitStr.length();
 
-		for (unsigned int i = 0; i < commitStrLen;) {
-			const unsigned int ucWidth = commitStr.at(i).isHighSurrogate() ? 2 : 1;
+		for (int i = 0; i < commitStrLen;) {
+			const int ucWidth = commitStr.at(i).isHighSurrogate() ? 2 : 1;
 			const QString oneCharUTF16 = commitStr.mid(i, ucWidth);
 			const QByteArray oneChar = sqt->BytesForDocument(oneCharUTF16);
 
-			sqt->InsertCharacter(oneChar.data(), oneChar.length(), EditModel::CharacterSource::directInput);
+			sqt->InsertCharacter(std::string_view(oneChar.data(), oneChar.length()), CharacterSource::DirectInput);
 			i += ucWidth;
 		}
 
 	} else if (!event->preeditString().isEmpty()) {
 		const QString preeditStr = event->preeditString();
-		const unsigned int preeditStrLen = preeditStr.length();
-		if ((preeditStrLen == 0) || (preeditStrLen > MAXLENINPUTIME)) {
+		const int preeditStrLen = preeditStr.length();
+		if (preeditStrLen == 0) {
 			sqt->ShowCaretAtCurrentPosition();
 			return;
 		}
@@ -573,15 +579,18 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 			sqt->ClearBeforeTentativeStart();
 		sqt->pdoc->TentativeStart(); // TentativeActive() from now on.
 
+		// Fix candidate window position at the start of preeditString.
+		preeditPos = sqt->CurrentPosition();
+
 		std::vector<int> imeIndicator = MapImeIndicators(event);
 
-		for (unsigned int i = 0; i < preeditStrLen;) {
-			const unsigned int ucWidth = preeditStr.at(i).isHighSurrogate() ? 2 : 1;
+		for (int i = 0; i < preeditStrLen;) {
+			const int ucWidth = preeditStr.at(i).isHighSurrogate() ? 2 : 1;
 			const QString oneCharUTF16 = preeditStr.mid(i, ucWidth);
 			const QByteArray oneChar = sqt->BytesForDocument(oneCharUTF16);
 			const int oneCharLen = oneChar.length();
 
-			sqt->InsertCharacter(oneChar.data(), oneCharLen, EditModel::CharacterSource::tentativeInput);
+			sqt->InsertCharacter(std::string_view(oneChar.data(), oneCharLen), CharacterSource::TentativeInput);
 
 			DrawImeIndicator(imeIndicator[i], oneCharLen);
 			i += ucWidth;
@@ -590,7 +599,7 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 		// Move IME carets.
 		int imeCaretPos = GetImeCaretPos(event);
 		int imeEndToImeCaretU16 = imeCaretPos - preeditStrLen;
-		int imeCaretPosDoc = sqt->pdoc->GetRelativePositionUTF16(sqt->CurrentPosition(), imeEndToImeCaretU16);
+		const Sci::Position imeCaretPosDoc = sqt->pdoc->GetRelativePositionUTF16(sqt->CurrentPosition(), imeEndToImeCaretU16);
 
 		MoveImeCarets(- sqt->CurrentPosition() + imeCaretPosDoc);
 
@@ -603,70 +612,68 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 #endif
 			sqt->view.imeCaretBlockOverride = true;
 		}
-
-		// Set candidate box position for Qt::ImMicroFocus.
-		preeditPos = sqt->CurrentPosition();
 		sqt->EnsureCaretVisible();
-		updateMicroFocus();
 	}
 	sqt->ShowCaretAtCurrentPosition();
 }
 
 QVariant ScintillaEditBase::inputMethodQuery(Qt::InputMethodQuery query) const
 {
-	int pos = send(SCI_GETCURRENTPOS);
-	int line = send(SCI_LINEFROMPOSITION, pos);
+	const Scintilla::Position pos = send(SCI_GETCURRENTPOS);
+	const Scintilla::Position line = send(SCI_LINEFROMPOSITION, pos);
 
 	switch (query) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+		// Qt 5 renamed ImMicroFocus to ImCursorRectangle then deprecated
+		// ImMicroFocus. Its the same value (2) and same description.
+		case Qt::ImCursorRectangle:
+		{
+			const Scintilla::Position startPos = (preeditPos >= 0) ? preeditPos : pos;
+			const Point pt = sqt->LocationFromPosition(startPos);
+			const int width = static_cast<int>(send(SCI_GETCARETWIDTH));
+			const int height = static_cast<int>(send(SCI_TEXTHEIGHT, line));
+			return QRectF(pt.x, pt.y, width, height).toRect();
+		}
+#else
 		case Qt::ImMicroFocus:
 		{
-			int startPos = (preeditPos >= 0) ? preeditPos : pos;
-			Point pt = sqt->LocationFromPosition(startPos);
-			int width = send(SCI_GETCARETWIDTH);
-			int height = send(SCI_TEXTHEIGHT, line);
+			const Scintilla::Position startPos = (preeditPos >= 0) ? preeditPos : pos;
+			const Point pt = sqt->LocationFromPosition(startPos);
+			const int width = static_cast<int>(send(SCI_GETCARETWIDTH));
+			const int height = static_cast<int>(send(SCI_TEXTHEIGHT, line));
 			return QRect(pt.x, pt.y, width, height);
 		}
+#endif
 
 		case Qt::ImFont:
 		{
 			char fontName[64];
-			int style = send(SCI_GETSTYLEAT, pos);
-			int len = send(SCI_STYLEGETFONT, style, (sptr_t)fontName);
-			int size = send(SCI_STYLEGETSIZE, style);
-			bool italic = send(SCI_STYLEGETITALIC, style);
-			int weight = send(SCI_STYLEGETBOLD, style) ? QFont::Bold : -1;
+			const sptr_t style = send(SCI_GETSTYLEAT, pos);
+			const int len = static_cast<int>(sends(SCI_STYLEGETFONT, style, fontName));
+			const int size = static_cast<int>(send(SCI_STYLEGETSIZE, style));
+			const bool italic = send(SCI_STYLEGETITALIC, style);
+			const int weight = send(SCI_STYLEGETBOLD, style) ? QFont::Bold : -1;
 			return QFont(QString::fromUtf8(fontName, len), size, weight, italic);
 		}
 
 		case Qt::ImCursorPosition:
 		{
-			int paraStart = sqt->pdoc->ParaUp(pos);
-			return pos - paraStart;
+			const Scintilla::Position paraStart = sqt->pdoc->ParaUp(pos);
+			return static_cast<int>(sqt->pdoc->CountUTF16(paraStart, pos));
 		}
 
 		case Qt::ImSurroundingText:
 		{
-			int paraStart = sqt->pdoc->ParaUp(pos);
-			int paraEnd = sqt->pdoc->ParaDown(pos);
-			QVarLengthArray<char,1024> buffer(paraEnd - paraStart + 1);
-
-			Sci_CharacterRange charRange;
-			charRange.cpMin = paraStart;
-			charRange.cpMax = paraEnd;
-
-			Sci_TextRange textRange;
-			textRange.chrg = charRange;
-			textRange.lpstrText = buffer.data();
-
-			send(SCI_GETTEXTRANGE, 0, (sptr_t)&textRange);
-
-			return sqt->StringFromDocument(buffer.constData());
+			const Scintilla::Position paraStart = sqt->pdoc->ParaUp(pos);
+			const Scintilla::Position paraEnd = sqt->pdoc->ParaDown(pos);
+			const std::string buffer = sqt->RangeText(paraStart, paraEnd);
+			return sqt->StringFromDocument(buffer.c_str());
 		}
 
 		case Qt::ImCurrentSelection:
 		{
-			QVarLengthArray<char,1024> buffer(send(SCI_GETSELTEXT));
-			send(SCI_GETSELTEXT, 0, (sptr_t)buffer.data());
+			QVarLengthArray<char,1024> buffer(send(SCI_GETSELTEXT)+1);
+			sends(SCI_GETSELTEXT, 0, buffer.data());
 
 			return sqt->StringFromDocument(buffer.constData());
 		}
@@ -676,57 +683,52 @@ QVariant ScintillaEditBase::inputMethodQuery(Qt::InputMethodQuery query) const
 	}
 }
 
-void ScintillaEditBase::notifyParent(SCNotification scn)
+void ScintillaEditBase::notifyParent(NotificationData scn)
 {
-	unsigned char updatedchar; //CodeQuery
 	emit notify(&scn);
 	switch (scn.nmhdr.code) {
-		case SCN_STYLENEEDED:
+		case Notification::StyleNeeded:
 			emit styleNeeded(scn.position);
 			break;
 
-		case SCN_CHARADDED:
+		case Notification::CharAdded:
 			emit charAdded(scn.ch);
 			break;
 
-		case SCN_SAVEPOINTREACHED:
+		case Notification::SavePointReached:
 			emit savePointChanged(false);
 			break;
 
-		case SCN_SAVEPOINTLEFT:
+		case Notification::SavePointLeft:
 			emit savePointChanged(true);
 			break;
 
-		case SCN_MODIFYATTEMPTRO:
+		case Notification::ModifyAttemptRO:
 			emit modifyAttemptReadOnly();
 			break;
 
-		case SCN_KEY:
+		case Notification::Key:
 			emit key(scn.ch);
 			break;
 
-		case SCN_DOUBLECLICK:
+		case Notification::DoubleClick:
 			emit doubleClick(scn.position, scn.line);
 			break;
 
-		case SCN_UPDATEUI:
-			emit updateUi(scn.updated);
-			// CodeQuery
-			updatedchar = (unsigned char) scn.updated;
-			/*static int i=0;
-			printf("Something happened %d ==> %d\n", i++, updatedchar);*/
-			if ((updatedchar & (SC_UPDATE_SELECTION)) == (SC_UPDATE_SELECTION))
-			{
-				emit selectionChanged(send(SCI_GETSELTEXT, 0, 0) > 1);
+		case Notification::UpdateUI:
+			if (FlagSet(scn.updated, Update::Selection)) {
+				updateMicroFocus();
+				emit selectionChanged(send(SCI_GETSELTEXT, 0, 0) > 1); //CodeQuery
 			}
+			emit updateUi(scn.updated);
 			break;
 
-		case SCN_MODIFIED:
+		case Notification::Modified:
 		{
-			bool added = scn.modificationType & SC_MOD_INSERTTEXT;
-			bool deleted = scn.modificationType & SC_MOD_DELETETEXT;
+			const bool added = FlagSet(scn.modificationType, ModificationFlags::InsertText);
+			const bool deleted = FlagSet(scn.modificationType, ModificationFlags::DeleteText);
 
-			int length = send(SCI_GETTEXTLENGTH);
+			const Scintilla::Position length = send(SCI_GETTEXTLENGTH);
 			bool firstLineAdded = (added && length == 1) ||
 			                      (deleted && length == 0);
 
@@ -736,74 +738,74 @@ void ScintillaEditBase::notifyParent(SCNotification scn)
 				emit linesAdded(added ? 1 : -1);
 			}
 
-			const QByteArray bytes = QByteArray::fromRawData(scn.text, scn.length);
+			const QByteArray bytes = QByteArray::fromRawData(scn.text, scn.text ? scn.length : 0);
 			emit modified(scn.modificationType, scn.position, scn.length,
 			              scn.linesAdded, bytes, scn.line,
 			              scn.foldLevelNow, scn.foldLevelPrev);
 			break;
 		}
 
-		case SCN_MACRORECORD:
+		case Notification::MacroRecord:
 			emit macroRecord(scn.message, scn.wParam, scn.lParam);
 			break;
 
-		case SCN_MARGINCLICK:
+		case Notification::MarginClick:
 			emit marginClicked(scn.position, scn.modifiers, scn.margin);
 			break;
 
-		case SCN_NEEDSHOWN:
+		case Notification::NeedShown:
 			emit needShown(scn.position, scn.length);
 			break;
 
-		case SCN_PAINTED:
+		case Notification::Painted:
 			emit painted();
 			break;
 
-		case SCN_USERLISTSELECTION:
+		case Notification::UserListSelection:
 			emit userListSelection();
 			break;
 
-		case SCN_URIDROPPED:
+		case Notification::URIDropped:
 			emit uriDropped(QString::fromUtf8(scn.text));
 			break;
 
-		case SCN_DWELLSTART:
+		case Notification::DwellStart:
 			emit dwellStart(scn.x, scn.y);
 			break;
 
-		case SCN_DWELLEND:
+		case Notification::DwellEnd:
 			emit dwellEnd(scn.x, scn.y);
 			break;
 
-		case SCN_ZOOM:
+		case Notification::Zoom:
 			emit zoom(send(SCI_GETZOOM));
 			break;
 
-		case SCN_HOTSPOTCLICK:
+		case Notification::HotSpotClick:
 			emit hotSpotClick(scn.position, scn.modifiers);
 			break;
 
-		case SCN_HOTSPOTDOUBLECLICK:
+		case Notification::HotSpotDoubleClick:
 			emit hotSpotDoubleClick(scn.position, scn.modifiers);
 			break;
 
-		case SCN_CALLTIPCLICK:
+		case Notification::CallTipClick:
 			emit callTipClick();
 			break;
 
-		case SCN_AUTOCSELECTION:
+		case Notification::AutoCSelection:
 			emit autoCompleteSelection(scn.lParam, QString::fromUtf8(scn.text));
 			break;
 
-		case SCN_AUTOCCANCELLED:
+		case Notification::AutoCCancelled:
 			emit autoCompleteCancelled();
 			break;
 
-		case SCN_FOCUSIN:
+		case Notification::FocusIn:
 			emit focusChanged(true);
 			break;
 
-		case SCN_FOCUSOUT:
+		case Notification::FocusOut:
 			emit focusChanged(false);
 			break;
 
@@ -817,11 +819,11 @@ void ScintillaEditBase::event_command(uptr_t wParam, sptr_t lParam)
 	emit command(wParam, lParam);
 }
 
-int ScintillaEditBase::ModifiersOfKeyboard() const
+KeyMod ScintillaEditBase::ModifiersOfKeyboard()
 {
 	const bool shift = QApplication::keyboardModifiers() & Qt::ShiftModifier;
 	const bool ctrl  = QApplication::keyboardModifiers() & Qt::ControlModifier;
 	const bool alt   = QApplication::keyboardModifiers() & Qt::AltModifier;
 
-	return ScintillaQt::ModifierFlags(shift, ctrl, alt);
+	return ModifierFlags(shift, ctrl, alt);
 }
