@@ -21,12 +21,13 @@
 
 //#define DRAW_AXIS
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-#define QT45_FOREGROUND(x) windowText(x)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#define LEVELOFDETAIL(p,x) p->levelOfDetailFromTransform(x)
 #else
-#define QT45_FOREGROUND(x) foreground(x)
+#define LEVELOFDETAIL(p,x) p->levelOfDetail
 #endif
 
+
 /** Constant for adjusting item's border rectangle */
 const qreal box_adjust = 5;
         
@@ -205,10 +206,11 @@ GNode::readFromElement( QDomElement e)
     {
         QString str = e.attribute( "label");
         item()->setPlainText( str);
-        QRegExp rx("(\\d+)");
-        if ( rx.indexIn( str) != -1)
+        QRegularExpression rx("(\\d+)");
+        auto pos = rx.match(str);
+        if (pos.hasMatch())
         {
-            setIRId( rx.cap( 1).toInt());
+            setIRId( pos.captured(1).toInt());
         }
     }
     if ( e.hasAttribute("type"))
@@ -484,8 +486,8 @@ NodeItem::paint( QPainter *painter,
         painter->drawLine( QPoint( -boundingRect().width(), 0), QPoint( boundingRect().width(),0));
         painter->drawLine( QPoint( 0, -boundingRect().height()), QPoint( 0, boundingRect().height()));
 #endif
-        QPen pen( option->palette.QT45_FOREGROUND().color(), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
-        if ( option->levelOfDetail < 0.1)
+        QPen pen( option->palette.windowText().color(), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
+        if ( LEVELOFDETAIL(option, painter->worldTransform()) < 0.1)
         {
             painter->fillRect( borderRect(), option->palette.highlight().color());
         }
@@ -518,14 +520,14 @@ NodeItem::paint( QPainter *painter,
                 painter->drawRect( borderRect());
             }
         }
-        if ( option->levelOfDetail >= 0.2)
+        if ( LEVELOFDETAIL(option, painter->worldTransform()) >= 0.2)
         {
             if ( painter->isActive())     
                 QGraphicsTextItem::paint( painter, option, widget);
         }
     } else if ( node()->isEdgeControl())
     {
-        if ( option->levelOfDetail < 0.2)
+        if ( LEVELOFDETAIL(option, painter->worldTransform()) < 0.2)
             return;
         if ( node()->firstPred()->item()->isSelected()
              || node()->firstSucc()->item()->isSelected())
@@ -533,11 +535,11 @@ NodeItem::paint( QPainter *painter,
             if ( bold_border && ( option->state & QStyle::State_Sunken)) 
             {
                 painter->setBrush( option->palette.highlight().color());
-                painter->setPen( QPen(option->palette.QT45_FOREGROUND().color(), 0));
+                painter->setPen( QPen(option->palette.windowText().color(), 0));
             } else
             {
                 painter->setBrush( option->palette.highlight().color());
-                painter->setPen( QPen(option->palette.QT45_FOREGROUND().color(), 0));
+                painter->setPen( QPen(option->palette.windowText().color(), 0));
             }
             painter->drawEllipse( -EdgeControlSize, -EdgeControlSize,
                                   2*EdgeControlSize, 2*EdgeControlSize);
